package com.myoa.engineering.crawl.shopping.event.handler;

import com.myoa.engineering.crawl.shopping.domain.model.UserNotifyModel;
import com.myoa.engineering.crawl.shopping.domain.model.v2.AppUserModel;
import com.myoa.engineering.crawl.shopping.domain.model.v2.ArticleModel;
import com.myoa.engineering.crawl.shopping.domain.model.v2.SubscribedKeywordAggregatedModel;
import com.myoa.engineering.crawl.shopping.event.ArticleUpsertEvent;
import com.myoa.engineering.crawl.shopping.service.AppUserQueryService;
import com.myoa.engineering.crawl.shopping.service.SubscribedKeywordCacheService;
import com.myoa.engineering.crawl.shopping.service.slack.SlackMessageBlockService;
import com.myoa.engineering.crawl.shopping.service.slack.UserNotifyService;
import com.myoa.engineering.crawl.shopping.support.dto.constant.CrawlTarget;
import com.myoa.engineering.crawl.shopping.util.SlackMessageUtils;
import com.slack.api.methods.request.chat.ChatPostMessageRequest;
import com.slack.api.methods.response.chat.ChatPostMessageResponse;
import com.slack.api.model.block.Blocks;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Component
public class ArticleUpsertEventListener {

    private final SubscribedKeywordCacheService subscribedKeywordCacheService;
    private final AppUserQueryService appUserQueryService;
    private final UserNotifyService userNotifyService;
    private final SlackMessageBlockService slackMessageBlockService;

    public ArticleUpsertEventListener(SubscribedKeywordCacheService subscribedKeywordCacheService,
                                      AppUserQueryService appUserQueryService,
                                      UserNotifyService userNotifyService,
                                      SlackMessageBlockService slackMessageBlockService) {
        this.subscribedKeywordCacheService = subscribedKeywordCacheService;
        this.appUserQueryService = appUserQueryService;
        this.userNotifyService = userNotifyService;
        this.slackMessageBlockService = slackMessageBlockService;
    }


    @EventListener
    public void handleArticleUpsertEvent(ArticleUpsertEvent event) {
        Map<CrawlTarget, List<ArticleModel>> articleMap =
                ((List<ArticleModel>) event.getSource()).stream()
                                                        .collect(Collectors.groupingBy(ArticleModel::getCrawlTarget));
        Map<CrawlTarget, ChatPostMessageResponse> allArticleNotifiedResultMap =
                articleMap.entrySet()
                          .stream()
                          .collect(Collectors.toMap(Map.Entry::getKey, e -> notifyMessage(e.getKey(), e.getValue())));

        List<AppUserModel> appUsers = appUserQueryService.findAll();

        appUsers.stream()
                .filter(AppUserModel::getEnabled)
                .flatMap(user -> {
                    Map<CrawlTarget, SubscribedKeywordAggregatedModel> subscribedKeywords =
                            subscribedKeywordCacheService.getSubscribedKeywordsCached(user.getSlackId());
                    return subscribedKeywords.entrySet()
                                             .stream()
                                             .map(entry -> {
                                                 List<ArticleModel> filtered = doAhocorasick(articleMap.get(entry.getKey()), entry.getValue());
                                                 return UserNotifyModel.builder()
                                                                       .slackId(user.getSlackId())
                                                                       .articles(filtered)
                                                                       .chatPostMessageResponse(allArticleNotifiedResultMap.get(entry.getKey()))
                                                                       .build();
                                             });
                })
                .forEach(this::notifyMessage);
    }

    private List<ArticleModel> doAhocorasick(List<ArticleModel> articles,
                                             SubscribedKeywordAggregatedModel trieModel) {
        if (articles == null || articles.isEmpty()) {
            return List.of();
        }
        return articles.stream()
                       .filter(article -> !trieModel.getAhoCorasickTrie()
                                                    .parseText(article.getTitle())
                                                    .isEmpty())
                       .toList();
    }

    private ChatPostMessageResponse notifyMessage(CrawlTarget crawlTarget, List<ArticleModel> articles) {
        String composited = articles.stream()
                                    .map(ArticleModel::convertArticletoMessage)
                                    .collect(Collectors.joining("\n"));

        ChatPostMessageRequest request =
                slackMessageBlockService.generateMessage()
                                        .blocks(Blocks.asBlocks(
                                                SlackMessageUtils.ofHeader(crawlTarget.getAlias()),
                                                SlackMessageUtils.ofSection(composited),
                                                SlackMessageUtils.ofDivider()
                                        ))
                                        .build();

        return userNotifyService.notify(request);
    }

    private void notifyMessage(UserNotifyModel userNotifyModel) {
        System.out.println("article = " + userNotifyModel);
        if (userNotifyModel.getArticles().isEmpty()) {
            return;
        }

        ChatPostMessageRequest request =
                slackMessageBlockService.generateMessage()
                                        .blocks(Blocks.asBlocks(
                                                SlackMessageUtils.ofSection(userNotifyModel.toCompositedMessage())
                                        ))
                                        .build();
        userNotifyService.notify(request);
    }

}
